/* umtsmon - a program to control/monitor your UMTS card
 * Copyright (C) 2006  Klaas van Gend
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * applicable version is GPL version 2 only.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "FileStuff.h"
#include <ntqdir.h>
#include <ntqstring.h>
#include <ntqregexp.h>

TQString
FileStuff::getFileFirstLine(const TQString& aFileName)
{
	TQFile myFile(aFileName);
	if (myFile.open(IO_ReadOnly))
	{
	   	TQTextStream in(&myFile);
		TQString myLine = in.readLine().stripWhiteSpace();
		myFile.close();
		return myLine;
	}
	return NULL;	
}

bool
FileStuff::doesFileContainString(
	const TQString& aFileName, 
	const TQRegExp& aSubString)
{
	TQFile myFile(aFileName);
	if (myFile.open(IO_ReadOnly) != true)
	{
		return false;
	}

	TQString myLine;
	bool myResult = false;
   	TQTextStream in(&myFile);
	while(!in.atEnd())
	{
		myLine = in.readLine().stripWhiteSpace();
		// perform substring searching
		if (aSubString.search(myLine,0) != -1)
		{
			myResult = true;
			break;	// breaks while loop
		}
	}
	myFile.close();
	return myResult;
}

bool 
FileStuff::doesFileExist(const TQString& aFileName)
{ 
	return TQFile::exists(aFileName);
}

FileStuff::ReadWrite 
FileStuff::isFileReadWrite(const TQString& aFileName)
{
	TQFile myFile(aFileName);
	if (myFile.open(IO_ReadWrite))
	{
		myFile.close();
		return READWRITE;
	}
	if (myFile.open(IO_ReadOnly))
	{
		myFile.close();
		return READ;
	}
	if (myFile.open(IO_WriteOnly))
	{
		myFile.close();
		return WRITE;
	}
	return NOTHING;
}


bool 
FileStuff::checkForRunningProcess(const TQString& aProcessName)
{
	// retrieve a sorted list of all entries in the proc filesystem
	TQString myBaseDirName="/proc/";
	TQDir myProcDir(myBaseDirName);
	myProcDir.setFilter( TQDir::Dirs );
	
	const TQFileInfoList* myList = myProcDir.entryInfoList();
	TQFileInfoListIterator myIterator( *myList );
	TQFileInfo* myFileInfo;
	
	while ( (myFileInfo = myIterator.current()) != 0 ) 
	{
		++myIterator;

		// if the dirname contains any non-number characters, including dots, skip it.
		TQRegExp myNonNumberRegExp("\\D");
		if (myFileInfo->fileName().contains(myNonNumberRegExp))
			continue;
		// and check for the ProcessName on the command line			
		TQString myCmdName = myBaseDirName + myFileInfo->fileName() + "/cmdline";
		if (FileStuff::doesFileContainString(myCmdName, aProcessName) == true)
			return true;
	}

	return false;
}



FileStuff::FindInDir::FindInDir(
			const TQString& aDirName, 
			const TQString& aNameFilter, 
			TQDir::FilterSpec aWhatEntriesToReturn)
{
	theDir.setPath(aDirName);
	theDir.setNameFilter(aNameFilter);
	theDir.setFilter(aWhatEntriesToReturn);
	
	theFIListPtr = theDir.entryInfoList();
	theFILIPtr = new TQFileInfoListIterator(*theFIListPtr);
}

FileStuff::FindInDir::~FindInDir()
{
	delete theFILIPtr;
}

TQString 
FileStuff::FindInDir::getNextEntry(void)
{
	TQFileInfo* myEntryInfoPtr = theFILIPtr->current();
	if (myEntryInfoPtr == NULL)
		return "";
	TQString myFileName = myEntryInfoPtr->absFilePath();
	++(*theFILIPtr);
	return myFileName;
}
