/* umtsmon - a program to control/monitor your UMTS card
 * This file copyright (C) 2006  Klaas van Gend
 *                     (C) 2007  Christofer Wesseling, Klaas van Gend
 *                     (C) 2008  Klaas van Gend
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * applicable version is GPL version 2 only.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#ifndef POPUP_H_
#define POPUP_H_

#include <ntqstring.h>
#include <ntqmessagebox.h>

// for exit(3)
#include <cstdlib>

/// DON'T use this function in your code!!!
extern void printAbortMessage(int aSignalNumber);


/** This is a class utility - it encapsulates three member functions to do 
 *  quick popup dialogs
 */
class Popup
{
public:
	/** pops up a message box with a critical message
	 *  PROGRAM EXECUTION WILL STOP AND THE APPLICATION WILL HALT
	 * @param aMessage: the Message
	 * @param currentWidget: the Widget below the Message - if Popup should appear in front of another form than the mainWindow
	 */
	static inline void Critical(const TQString& anErrorMessage, TQWidget* currentWidget = NULL)
	{
		DEBUG1("*** CRITICAL ERROR: %s\n",anErrorMessage.ascii());
		TQMessageBox::critical (currentWidget, 
				APPNAME " critical error",
				anErrorMessage,
				TQMessageBox::Abort |  TQMessageBox::Default, 
				TQMessageBox::NoButton, 
				TQMessageBox::NoButton);
		// critical errors? we might want to explain the user on 
		// how to help solve this!
		printAbortMessage(0);
		// printAbortMessage doesn't exit - good!
		// leaves us room to explain why we actually stop
		DEBUG1("Critical Popup - application will halt\n");
		exit(10);	
	};
	
	/** pops up a message box with a informational message
	 *  program execution will continue afterwards
	 * @param aMessage: the Message
	 * @param currentWidget: the Widget below the Message - if Popup should appear in front of another form than the mainWindow 
	 */
	static inline void Info(const TQString& aMessage, TQWidget* currentWidget = NULL)
	{
		DEBUG2("Info Message: %s\n",aMessage.ascii());
		TQMessageBox::information (currentWidget, 
				APPNAME " informational message",
				aMessage,
				TQMessageBox::Ok |  TQMessageBox::Default, 
				TQMessageBox::NoButton, 
				TQMessageBox::NoButton);
	};
	
	/** pops up a message box with a warning message
	 *  program execution will continue afterwards
	 * @param aMessage: the Message
	 * @param currentWidget: the Widget below the Message - if Popup should appear in front of another form than the mainWindow
	 */
	static inline void Warning(const TQString& aMessage, TQWidget* currentWidget = NULL)
	{
		DEBUG2("Warning Message: %s\n",aMessage.ascii());
		TQMessageBox::warning (currentWidget, 
				APPNAME " warning",
				aMessage,
				TQMessageBox::Ok |  TQMessageBox::Default, 
				TQMessageBox::NoButton, 
				TQMessageBox::NoButton);
	};

	/** pops up a message box with a question message
	 *  if question answered YES, will return true, otherwise false
	 * @param aMessage: the Message
	 * @param currentWidget: the Widget below the Message - if Popup should appear in front of another form than the mainWindow
	 * @returns true if clicked Yes, false if no or 'escape'
	 */
	static inline bool YesNoQuestion(const TQString& aMessage, TQWidget* currentWidget = NULL)
	{
		DEBUG2("Yes/No Question: %s\n", aMessage.ascii());
		if (TQMessageBox::question(currentWidget, 
				APPNAME " question",
				aMessage,
				TQMessageBox::Yes | TQMessageBox::Default, 
				TQMessageBox::No, 
				TQMessageBox::NoButton) == TQMessageBox::Yes)
			return true;
		else
			return false;
	};
	
	
	/** pops up a message box with a warning message
	 *  program execution will continue afterwards
	 *  the message box will contain a checkbox "do not show again", which can be
	 *  checked - and works as advertised :-)
	 * @param aName: unique name of the message which will be used to save the user's preference
	 * @param aMessage: the Message
	 * @param currentWidget: the Widget below the Message - if Popup should appear in front of another form than the mainWindow
	 */
	static void WarningWithMemory(const TQString& aName, const TQString& aMessage, TQWidget* currentWidget = NULL);

	/** pops up a message box with a question message
	 *  if question answered YES, will return true, otherwise false
	 *  the message box will contain a checkbox "do not show again", which can be
	 *  checked - and works as advertised :-)
	 * @param aName: unique name of the message which will be used to save the user's preference
	 * @param aMessage: the Message
	 * @param currentWidget: the Widget below the Message - if Popup should appear in front of another form than the mainWindow
	 * @returns true if clicked Yes, false if no or 'escape'
	 */
	static bool YesNoQuestionWithMemory(const TQString& aName, const TQString& aMessage, TQWidget* currentWidget = NULL);
	
}; 


#endif /*POPUP_H_*/
